;
; cat.asm
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;; cat [files]
;;
;;   Reads binary files and outputs them. Can be used to concatenate
;;   multiple files.
;;
;    This file is part of the IDE64 shell utilities.
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

WILDEXP		= true

INPUT_LFN	= 1

bufp		= $fd
buffsize	= $4000

banner		= "cat 0.3 by soci/singular"

		.include "lib.asm"

		.section code
start2		.proc
		ldx #<arglist
		ldy #>arglist
		jsr parseargs
		lda #<arglist
		sta p1
		lda #>arglist
		sta p1+1

		lda #0
		sta write.unsupported

		jsr resetbufp

loop		ldy #0
		lda (p1),y
		beq _exit

		ldx p1
		ldy p1+1
		inx
		bne +
		iny
+		jsr kernal.setnam
		jsr addstring
		lda #INPUT_LFN
		ldx mydrive
		ldy #0
		sty read.unsupported
		jsr kernal.setlfs
		jsr kernal.open
		bcs _nopen
		jsr onefile
		jsr kernal.clrchn
		lda #INPUT_LFN
		jsr kernal.close
_nopen		ldx kernal.stop
		cpx #$7f
		bne loop
_exit		jsr flush
		jsr kernal.clrchn
		jmp exit

onefile		.proc
		jsr calcfree
		bne _notnull
		txa
		bne _notnull
		jsr flush
		bcs _err
_notnull
		ldx #INPUT_LFN
		stx myin
		jsr kernal.chkin
		bcs _err

		jsr calcfree
		lda #bufp
		jsr read
		bcs _err

		txa
		adc bufp
		sta bufp
		tya
		adc bufp+1
		sta bufp+1
		lda kernal.status
		beq onefile
_err		rts
		.pend

flush		.proc
		jsr calcused
		ora bufp
		bne +
err		rts
+
		ldx #STDOUT_LFN
		stx myout
		jsr kernal.chkout
		bcs err

		jsr calcused
		jsr resetbufp
		lda #bufp
		jmp write
		.pend

calcfree	.proc
		lda #<buffer+size(buffer)
		sec
		sbc bufp
		tax
		lda #>buffer+size(buffer)
		sbc bufp+1
		tay
		rts
		.pend

calcused	.proc
		ldx bufp
		lda bufp+1
		sec
		sbc #>buffer
		tay
		rts
		.pend

resetbufp	.proc
		lda #<buffer
		sta bufp
		lda #>buffer
		sta bufp+1
		rts
		.pend

		.section bss
		.align 256
buffer		.fill buffsize
		.send bss
		.pend
		.send code

