;
; md5sum.asm
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;; md5sum [files]
;;
;;   Calculates md5 hash on files as described in RFC 1321, and prints
;;   the result. It can be used for file integrity checking, better then
;;   cksum, but much slower.
;;
;    This file is part of the IDE64 shell utilities.
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

WILDEXP		= true

INPUT_LFN	= 1

		*= $2
		.dsection zp

zps		.section zp
aa		.fill 4
bb		.fill 4
cc		.fill 4
dd		.fill 4
buf		.fill 64
bufp		.word ?
ichars		.byte ?
xx		.byte ?
filelen		.dword ?
		.send zp

banner		= "md5sum 0.2 by soci/singular"

		.include "lib.asm"

		.section code
start2		.proc
		ldx #<arglist
		ldy #>arglist
		jsr parseargs
		lda #<arglist
		sta p1
		lda #>arglist
		sta p1+1

loop		ldy #0
		lda (p1),y
		beq _exit

		ldx p1
		ldy p1+1
		inx
		bne +
		iny
+		jsr kernal.setnam
		jsr addstring
		lda #INPUT_LFN
		ldx mydrive
		ldy #0
		jsr kernal.setlfs
		jsr kernal.open
		bcs _nopen
		jsr onefile
		jsr kernal.clrchn
		lda #INPUT_LFN
		jsr kernal.close
_nopen		ldx kernal.stop
		cpx #$7f
		bne loop
_exit		jmp exit

onefile		.proc
		jsr zpsave
		ldx #15
-		lda initval,x
		sta result,x
		sta aa,x
		cpx #4
		bge +
		lda #0
		sta xx
		sta filelen,x
+		dex
		bpl -

		jsr in
		jsr zprestore

		bcc +
err		rts
+
		ldx #STDOUT_LFN
		jsr kernal.chkout
		bcs err

		ldx #-16
-		lda result-(<-16),x
		jsr printhex
		inx
		bne -
		lda #32
		jsr kernal.chrout
		jsr kernal.chrout
		ldy #0
-		cpy kernal.filename_len
		beq +
		lda (kernal.filename),y
		jsr kernal.chrout
		iny
		bne -
+		lda #"{cr}"
		jmp kernal.chrout
		.pend

in		.proc
		ldx #INPUT_LFN
		stx myin
		jsr kernal.chkin
		bcc next
err		rts

next
		lda #<buffer
		sta bufp
		lda #>buffer
		sta bufp+1

		ldx #<size(buffer)
		ldy #>size(buffer)
		lda #bufp
		jsr read
		bcs err
		txa
		adc filelen
		sta filelen
                tya
		adc filelen+1
		sta filelen+1
                bcc +
		inc filelen+2
                bne +
		inc filelen+3
+
		stx ichars
		tya
		asl ichars
		rol a
		asl ichars
		rol a
		beq ur
-		pha
		jsr md5
		lda bufp
		clc
		adc #64
		sta bufp
		bcc +
		inc bufp+1
+		pla
		sec
		sbc #1
		bne -
ur
		lda kernal.status
		beq next

		lda ichars
		lsr a
		lsr a
		pha
		tay
		lda #$80
-		sta (bufp),y
		lda #0
		iny
		cpy #64
		blt -
		pla
		cmp #56
		blt +
		jsr md5
		ldy #63
		lda #0
-		sta (bufp),y
		dey
		bpl -

+		ldy #56
-		lda filelen-56,y
		sta (bufp),y
		iny
		cpy #60
		blt -
		ldx #3
-		ldy #56
		clc
-		lda (bufp),y
		rol a
		sta (bufp),y
		iny
		tya
		eor #61
		bne -
		dex
		bne --
		jsr md5
		clc
		rts
		.pend

shift		.macro
		.if \2==4
		asl a			; << 4
		rol \1
		rol \1+1
		rol \1+2
		adc #$80
		rol \1
		rol \1+1
		rol \1+2
		rol a
		asl a
		rol \1
		rol \1+1
		rol \1+2
		adc #$80
		rol \1
		rol \1+1
		rol \1+2
		rol a
		sta \1+3
		.elsif \2==5
		tay			; << 5
		lsr a
		lda \1+2
		ror a
		sta \1+3
		lda \1+1
		ror a
		sta \1+2
		lda \1
		ror a
		sta \1+1
		tya
		ror a
		lsr a
		ror \1+3
		ror \1+2
		ror \1+1
		bcc +
		ora #$80
+		lsr a
		ror \1+3
		ror \1+2
		ror \1+1
		bcc +
		ora #$80
+		sta \1
		.elsif \2==6
		tay			; << 6
		lsr a
		lda \1+2
		ror a
		sta \1+3
		lda \1+1
		ror a
		sta \1+2
		lda \1
		ror a
		sta \1+1
		tya
		ror a
		lsr a
		ror \1+3
		ror \1+2
		ror \1+1
		bcc +
		ora #$80
+		sta \1
		.elsif \2==7
		tay			; << 7
		lsr a
		lda \1+2
		ror a
		sta \1+3
		lda \1+1
		ror a
		sta \1+2
		lda \1
		ror a
		sta \1+1
		tya
		ror a
		sta \1
		.elsif \2==9
		tay			; << 9
		asl a
		lda \1
		rol a
		ldx \1+1
		sta \1+1
		txa
		rol a
		ldx \1+2
		sta \1+2
		txa
		rol a
		sta \1+3
		tya
		rol a
		sta \1
		.elsif \2==10
		asl a			; << 10
		tay
		lda \1
		rol a
		ldx \1+1
		sta \1+1
		txa
		rol a
		ldx \1+2
		sta \1+2
		txa
		rol a
		sta \1+3
		tya
		adc #$80
		rol \1+1
		rol \1+2
		rol \1+3
		rol a
		sta \1
		.elsif \2==11
		tay			; << 11
		asl a
		lda \1
		rol a
		ldx \1+1
		sta \1+1
		txa
		rol a
		ldx \1+2
		sta \1+2
		txa
		rol a
		sta \1+3
		tya
		rol a
		asl a
		rol \1+1
		rol \1+2
		rol \1+3
		adc #$80
		rol \1+1
		rol \1+2
		rol \1+3
		rol a
		sta \1
		.elsif \2==12
		asl a			; << 12
		tay
		lda \1
		rol a
		ldx \1+1
		sta \1+1
		txa
		rol a
		ldx \1+2
		sta \1+2
		txa
		rol a
		sta \1+3
		tya
		adc #$80
		rol \1+1
		rol \1+2
		rol \1+3
		rol a
		asl a
		rol \1+1
		rol \1+2
		rol \1+3
		adc #$80
		rol \1+1
		rol \1+2
		rol \1+3
		rol a
		sta \1
		.elsif \2==14
		tay			; << 14
		lsr a
		lda \1+2
		ror a
		ldx \1
		sta \1
		lda \1+1
		ror a
		sta \1+3
		txa
		ror a
		sta \1+2
		tya
		ror a
		lsr a
		ror \1
		ror \1+3
		ror \1+2
		bcc +
		ora #$80
+		sta \1+1
		.elsif \2==15
		tay			; << 15
		lsr a
		lda \1+2
		ror a
		ldx \1
		sta \1
		lda \1+1
		ror a
		sta \1+3
		txa
		ror a
		sta \1+2
		tya
		ror a
		sta \1+1
		.elsif \2==16
		ldx \1+1		; << 16
		sta \1+1
		stx \1+3
		lda \1
		ldx \1+2
		sta \1+2
		stx \1
		.elsif \2==17
		tay			; << 17
		asl a
		lda \1
		rol a
		ldx \1+2
		sta \1+2
		lda \1+1
		rol a
		sta \1+3
		txa
		rol a
		sta \1
		tya
		rol a
		sta \1+1
		.elsif \2==20
		asl a			; << 20
		tay
		lda \1
		rol a
		ldx \1+2
		sta \1+2
		lda \1+1
		rol a
		sta \1+3
		txa
		rol a
		sta \1
		tya
		adc #$80
		rol \1+2
		rol \1+3
		rol \1
		rol a
		asl a
		rol \1+2
		rol \1+3
		rol \1
		adc #$80
		rol \1+2
		rol \1+3
		rol \1
		rol a
		sta \1+1
		.elsif \2==21
		tay			; << 21
		lsr a
		lda \1+2
		ror a
		ldx \1+1
		sta \1+1
		txa
		ror a
		ldx \1
		sta \1
		txa
		ror a
		sta \1+3
		tya
		ror a
		lsr a
		ror \1+1
		ror \1
		ror \1+3
		bcc +
		ora #$80
+		lsr a
		ror \1+1
		ror \1
		ror \1+3
		bcc +
		ora #$80
+		sta \1+2
		.elsif \2==22
		tay			; << 22
		lsr a
		lda \1+2
		ror a
		ldx \1+1
		sta \1+1
		txa
		ror a
		ldx \1
		sta \1
		txa
		ror a
		sta \1+3
		tya
		ror a
		lsr a
		ror \1+1
		ror \1
		ror \1+3
		bcc +
		ora #$80
+		sta \1+2
		.elsif \2==23
		tay			; << 23
		lsr a
		lda \1+2
		ror a
		ldx \1+1
		sta \1+1
		txa
		ror a
		ldx \1
		sta \1
		txa
		ror a
		sta \1+3
		tya
		ror a
		sta \1+2
		.else
		.error \2
		.fi
		.endm

stepf		.macro
		clc
		lda \4+0
		eor \3+0
		and \2+0
		eor \4+0
		adc \1+0
		rol xx
		adc buf+\5*4
		rol xx
		adc #<(\6)
		sta \1+0

		lda \4+1
		eor \3+1
		and \2+1
		eor \4+1
		adc \1+1
		ror xx
		adc buf+\5*4+1
		ror xx
		adc #>(\6)
		sta \1+1

		lda \4+2
		eor \3+2
		and \2+2
		eor \4+2
		adc \1+2
		rol xx
		adc buf+\5*4+2
		rol xx
		adc #<((\6) >> 16)
		sta \1+2

		lda \4+3
		eor \3+3
		and \2+3
		eor \4+3
		adc \1+3
		lsr xx
		adc buf+\5*4+3
		lsr xx
		adc #>((\6) >> 16)

		#shift \1, \7
		clc
		.for i=0,i<4,i=i+1
		lda \1+i
		adc \2+i
		sta \1+i
		.next
		.endm

stepg		.macro
		clc
		lda \3+0
		eor \2+0
		and \4+0
		eor \3+0
		adc \1+0
		rol xx
		adc buf+\5*4
		rol xx
		adc #<(\6)
		sta \1+0

		lda \3+1
		eor \2+1
		and \4+1
		eor \3+1
		adc \1+1
		ror xx
		adc buf+\5*4+1
		ror xx
		adc #>(\6)
		sta \1+1

		lda \3+2
		eor \2+2
		and \4+2
		eor \3+2
		adc \1+2
		rol xx
		adc buf+\5*4+2
		rol xx
		adc #<((\6) >> 16)
		sta \1+2

		lda \3+3
		eor \2+3
		and \4+3
		eor \3+3
		adc \1+3
		lsr xx
		adc buf+\5*4+3
		lsr xx
		adc #>((\6) >> 16)

		#shift \1, \7
		clc
		.for i=0,i<4,i=i+1
		lda \1+i
		adc \2+i
		sta \1+i
		.next
		.endm

steph		.macro
		clc
		lda \2+0
		eor \3+0
		eor \4+0
		adc \1+0
		rol xx
		adc buf+\5*4
		rol xx
		adc #<(\6)
		sta \1+0

		lda \2+1
		eor \3+1
		eor \4+1
		adc \1+1
		ror xx
		adc buf+\5*4+1
		ror xx
		adc #>(\6)
		sta \1+1

		lda \2+2
		eor \3+2
		eor \4+2
		adc \1+2
		rol xx
		adc buf+\5*4+2
		rol xx
		adc #<((\6) >> 16)
		sta \1+2

		lda \2+3
		eor \3+3
		eor \4+3
		adc \1+3
		lsr xx
		adc buf+\5*4+3
		lsr xx
		adc #>((\6) >> 16)

		#shift \1, \7
		clc
		.for i=0,i<4,i=i+1
		lda \1+i
		adc \2+i
		sta \1+i
		.next
		.endm

stepi		.macro
		clc
		lda \4+0
		eor #255
		ora \2+0
		eor \3+0
		adc \1+0
		rol xx
		adc buf+\5*4
		rol xx
		adc #<(\6)
		sta \1+0

		lda \4+1
		eor #255
		ora \2+1
		eor \3+1
		adc \1+1
		ror xx
		adc buf+\5*4+1
		ror xx
		adc #>(\6)
		sta \1+1

		lda \4+2
		eor #255
		ora \2+2
		eor \3+2
		adc \1+2
		rol xx
		adc buf+\5*4+2
		rol xx
		adc #<((\6) >> 16)
		sta \1+2

		lda \4+3
		eor #255
		ora \2+3
		eor \3+3
		adc \1+3
		lsr xx
		adc buf+\5*4+3
		lsr xx
		adc #>((\6) >> 16)

		#shift \1, \7
		clc
		.for i=0,i<4,i=i+1
		lda \1+i
		adc \2+i
		sta \1+i
		.next
		.endm

md5		.proc
		ldy #63
-		lda (bufp),y
		sta buf,y
		dey
		bpl -

		#stepf aa, bb, cc, dd, 0, -680876936, 7
		#stepf dd, aa, bb, cc, 1, -389564586, 12
		#stepf cc, dd, aa, bb, 2, $242070db, 17
		#stepf bb, cc, dd, aa, 3, -1044525330, 22
		#stepf aa, bb, cc, dd, 4, -176418897, 7
		#stepf dd, aa, bb, cc, 5, $4787c62a, 12
		#stepf cc, dd, aa, bb, 6, -1473231341, 17
		#stepf bb, cc, dd, aa, 7, -45705983, 22
		#stepf aa, bb, cc, dd, 8, $698098d8, 7
		#stepf dd, aa, bb, cc, 9, -1958414417, 12
		#stepf cc, dd, aa, bb, 10, -42063, 17
		#stepf bb, cc, dd, aa, 11, -1990404162, 22
		#stepf aa, bb, cc, dd, 12, $6b901122, 7
		#stepf dd, aa, bb, cc, 13, -40341101, 12
		#stepf cc, dd, aa, bb, 14, -1502002290, 17
		#stepf bb, cc, dd, aa, 15, $49b40821, 22

		#stepg aa, bb, cc, dd, 1, -165796510, 5
		#stepg dd, aa, bb, cc, 6, -1069501632, 9
		#stepg cc, dd, aa, bb, 11, $265e5a51, 14
		#stepg bb, cc, dd, aa, 0, -373897302, 20
		#stepg aa, bb, cc, dd, 5, -701558691, 5
		#stepg dd, aa, bb, cc, 10, $02441453, 9
		#stepg cc, dd, aa, bb, 15, -660478335, 14
		#stepg bb, cc, dd, aa, 4, -405537848, 20
		#stepg aa, bb, cc, dd, 9, $21e1cde6, 5
		#stepg dd, aa, bb, cc, 14, -1019803690, 9
		#stepg cc, dd, aa, bb, 3, -187363961, 14
		#stepg bb, cc, dd, aa, 8, $455a14ed, 20
		#stepg aa, bb, cc, dd, 13, -1444681467, 5
		#stepg dd, aa, bb, cc, 2, -51403784, 9
		#stepg cc, dd, aa, bb, 7, $676f02d9, 14
		#stepg bb, cc, dd, aa, 12, -1926607734, 20

		#steph aa, bb, cc, dd, 5, -378558, 4
		#steph dd, aa, bb, cc, 8, -2022574463, 11
		#steph cc, dd, aa, bb, 11, $6d9d6122, 16
		#steph bb, cc, dd, aa, 14, -35309556, 23
		#steph aa, bb, cc, dd, 1, -1530992060, 4
		#steph dd, aa, bb, cc, 4, $4bdecfa9, 11
		#steph cc, dd, aa, bb, 7, -155497632, 16
		#steph bb, cc, dd, aa, 10, -1094730640, 23
		#steph aa, bb, cc, dd, 13, $289b7ec6, 4
		#steph dd, aa, bb, cc, 0, -358537222, 11
		#steph cc, dd, aa, bb, 3, -722521979, 16
		#steph bb, cc, dd, aa, 6, $04881d05, 23
		#steph aa, bb, cc, dd, 9, -640364487, 4
		#steph dd, aa, bb, cc, 12, -421815835, 11
		#steph cc, dd, aa, bb, 15, $1fa27cf8, 16
		#steph bb, cc, dd, aa, 2, -995338651, 23

		#stepi aa, bb, cc, dd, 0, -198630844, 6
		#stepi dd, aa, bb, cc, 7, $432aff97, 10
		#stepi cc, dd, aa, bb, 14, -1416354905, 15
		#stepi bb, cc, dd, aa, 5, -57434055, 21
		#stepi aa, bb, cc, dd, 12, $655b59c3, 6
		#stepi dd, aa, bb, cc, 3, -1894986606, 10
		#stepi cc, dd, aa, bb, 10, -1051523, 15
		#stepi bb, cc, dd, aa, 1, -2054922799, 21
		#stepi aa, bb, cc, dd, 8, $6fa87e4f, 6
		#stepi dd, aa, bb, cc, 15, -30611744, 10
		#stepi cc, dd, aa, bb, 6, -1560198380, 15
		#stepi bb, cc, dd, aa, 13, $4e0811a1, 21
		#stepi aa, bb, cc, dd, 4, -145523070, 6
		#stepi dd, aa, bb, cc, 11, -1120210379, 10
		#stepi cc, dd, aa, bb, 2, $2ad7d2bb, 15
		#stepi bb, cc, dd, aa, 9, -343485551, 21

		clc
		.for i in range(4)
		lda result+i
		adc aa+i
		sta aa+i
		sta result+i
		.next
		clc
		.for i in range(4)
		lda result+i+4
		adc bb+i
		sta bb+i
		sta result+i+4
		.next
		clc
		.for i in range(4)
		lda result+i+8
		adc cc+i
		sta cc+i
		sta result+i+8
		.next
		clc
		.for i in range(4)
		lda result+i+12
		adc dd+i
		sta dd+i
		sta result+i+12
		.next
		rts
		.pend

zpsave		.proc
		.with zprestore
		lda $300
		sta o300+1
		lda $301
		sta o301+1
		ldx #+size(zps)-1
-		lda zps,x
		sta zpstore,x
		dex
		bpl -
		.endwith
		lda #<exithack
		sta $300
		lda #>exithack
		sta $301
		rts

exithack	txa
		jsr zprestore
		tax
		jmp ($300)
		.pend

zprestore	.proc
		ldx #+size(zps)-1
-		ldy zpstore,x
		sty zps,x
		dex
		bpl -
o300		ldx #0
		stx $300
o301		ldx #0
		stx $301
		rts

		.section bss
zpstore		.fill size(zps)
		.send bss
		.pend

initval		.section data
		.byte $01,$23,$45,$67
		.byte $89,$ab,$cd,$ef
		.byte $fe,$dc,$ba,$98
		.byte $76,$54,$32,$10
		.send data

		.section bss
result		.fill 16

		.align 256
buffer		.fill $3000
		.send bss

		.pend
		.send code
